# $Id: event.rb,v 1.1 2004/04/20 23:03:01 ianmacd Exp $

require "ctime/attendee"
require "date"


class Event

  attr_accessor :uid
  attr_accessor :summary
  attr_accessor :location
  attr_accessor :description
  attr_accessor :attendees
  attr_accessor :organizer

  def initialize(start_time, duration, summary, location, description)
    @attendees = Array.new
    @start_time = start_time
    @end_time = nil
    @duration = duration
    @summary = summary
    @location = location
    @description = description
  end

  def start_time=(startTime)
    if startTime.kind_of? String
      @start_time = parse_date_time(startTime)
    elsif startTime.kind_of? Time
      @start_time = startTime
    end
  end

  def start_time()
    @start_time
  end

  def start_time_str()
    start_time = start_time()
    sprintf("%s%02d%02dT%02d%02d00Z", start_time.year,
            start_time.month, start_time.day,
            start_time.hour, start_time.min)
  end


  def end_time=(endTime)
    if endTime.kind_of? String
      @end_time = parse_date_time(endTime)
    elsif endTime.kind_of? Time
      @end_time = endTime
    end
  end

  def end_time()
    if @end_time
      @end_time
    elsif @start_time && @duration
      @end_time = @start_time + @duration
    else
      nil
    end
  end

  def end_time_str()
    end_time = end_time()
    sprintf("%s%02d%02dT%02d%02d00Z", end_time.year,
            end_time.month, end_time.day,
            end_time.hour, end_time.min)
  end


  def duration=(duration)
    if duration.kind_of? String
      @duration = parse_duration(duration)
    elsif duration.kind_of? Integer
      @duration = duration
    end
  end

  def duration()
    if @duration
      @duration
    elsif @start_time && @end_time
      @duration = (@end_time - @start_time).to_i
    else
      nil
    end
  end


  def inspect
    result = "Event\n" +
             "\tuid=#{uid()}\n" +
             "\tsummary=#{summary()}\n" +
             "\tlocation=#{location()}\n" +
             "\tdescription=#{description()}\n" +
             "\tstart_time=#{start_time()}\n" +
             "\tend_time=#{end_time()}\n" +
             "\tduration=#{duration()}\n"
    @attendees.each do |attendee|
      result += attendee.inspect
    end

    result
  end


  #
  # Parses an ICal Date-Time value.  Returns a Ruby Time.
  #
  # Currently it only recognizes Date With UTC and Date With Local Time
  # formats.
  #
  def parse_date_time(dateTimeStr)
    if dateTimeStr =~ /^(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)Z$/
      return Time.gm($1, $2, $3, $4, $5, $6).localtime()
    elsif dateTimeStr =~ /^(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)$/
      return Time.local($1, $2, $3, $4, $5, $6)
    else
      return nil
    end
  end

  #
  # Parses an ICal Duration value.  Returns the value a integer number of
  # seconds.
  #
  def parse_duration(duration)
    weeks = 0
    days = 0
    hours = 0
    mins = 0
    secs = 0

    duration.sub!(/^[-+]*P/, "")

    duration.sub!(/^(\d+)W/) do |match|
      weeks = $1.to_s.to_i
      ""
    end

    duration.sub!(/^(\d+)D/) do |match|
      days = $1.to_s.to_i
      ""
    end

    duration.sub!(/^T/, "")

    duration.sub!(/^(\d+)H/) do |match|
      hours = $1.to_s.to_i
      ""
    end

    duration.sub!(/^(\d+)M/) do |match|
      mins = $1.to_s.to_i
      ""
    end

    duration.sub!(/^(\d+)S/) do |match|
      secs = $1.to_s.to_i
      ""
    end

    return (weeks * 7 * 24 * 3600) +
           (days * 24 * 3600) +
           (hours * 3600) +
           (mins * 60) +
           secs
  end


  def Event.parse_events(rawEventString)
    events = Array.new
    insideEvent = false
    name = ""
    paramHash = {}
    value = ""
    event = nil

    if $DEBUG
      puts "Raw string:"
      puts rawEventString
      puts "---------"
    end

    #
    # Get rid of folded lines.
    #
    # I don't see where this is in the spec, but Steltor also
    # "continues" lines by ending them with =\r\n
    #
    rawEventString.gsub!(/\r\n[ \t]/, "")
    rawEventString.gsub!(/=\r\n/, "")

    rawEventString.each_line do |line|
      (name, paramHash, value) = Event.parse_line(line)
      next if name == ""

      if $DEBUG
        puts "Name: #{name}   Value: #{value}"
        paramHash.each_key do |key|
            puts "\t#{key}=#{paramHash[key]}"
        end
      end

      #
      # Parse out events
      #
      if ! insideEvent
        if (name == "BEGIN") && (value == "VEVENT")
          insideEvent = true
          event = Event.new(nil, nil, nil, nil, nil)
          events.push(event)
        end
      else
        case name
          when "UID"
            event.uid = value
          when "DTSTART"
            event.start_time = value
            if paramHash["VALUE"] == "DATE"
              event.end = value
            end
          when "DTEND"
            event.end_time = value
          when "DURATION"
            event.duration = value
          when "SUMMARY"
            event.summary = value
          when "LOCATION"
            event.location = value
          when "DESCRIPTION"
            event.description = value
          when "ATTENDEE"
            event.attendees.push(Attendee.parse_attendee(paramHash, value))
          when "ORGANIZER"
            event.organizer = value.sub(/^mailto:/i, "")
          when "END"
            if value == "VEVENT"
              insideEvent = false
              event = nil
            end
        else
          puts "Unmatched name *#{name}*" if $DEBUG
        end
      end
    end

    return events
  end


  def Event.parse_line(line)
    name = ""
    value = ""
    paramHash = Hash.new

    line.sub!(/\r\n$/, "")

    if ( line =~ /^([^:]*):(.*)$/ )
      name = $1
      value = $2
      if ( name =~ /^([^;]*);(.*)$/ )
        name = $1
        paramString = $2.unpack("M*").to_s
        paramString.split(/;/).each do |nameValueString|
          if ( nameValueString =~ /^([^=]*)=(.*)$/ )
            paramHash[$1] = $2
          end
        end
      end

      name = name.unpack("M*").to_s
      value = value.unpack("M*").to_s
    end

    return [name, paramHash, value]
  end

end
